struct PSInput
{
	float4 position : SV_POSITION;
	float2 uv : TEXCOORD;
	float4 color : COLOR;
};

Texture2D g_texture : register(t0);
Texture2D g_textureNormalWorld : register(t1);
Texture2D g_textureOtherWorld : register(t2);
SamplerState g_sampler : register(s0);

cbuffer cb0 : register(b0)
{
	float4x4 mvMat;
	float4x4 projMat;
};

PSInput VSMain(float2 position : POSITION, float2 uv : TEXCOORD, float4 color : COLOR)
{
	PSInput result;

	float4x4 finalMatrix = mul(projMat, mvMat);
	result.position = mul(finalMatrix, float4(position, 0.0, 1.0f));
	result.uv = uv;
	result.color = color;

	return result;
}

float4 PSMain(PSInput input) : SV_TARGET
{
	float4 outColor;
	float ratio = g_texture.Sample(g_sampler, input.uv).r;
	if(ratio == 1.0f)
		outColor = g_textureNormalWorld.Sample(g_sampler, input.uv);
	else if(ratio == 0.0f)
		outColor = g_textureOtherWorld.Sample(g_sampler, input.uv);
	else
	{
		float4 color1 = g_textureNormalWorld.Sample(g_sampler, input.uv);
		float4 color2 = g_textureOtherWorld.Sample(g_sampler, input.uv);
		outColor = lerp(color1, color2, 1-ratio);
	}
	return outColor;
}
