#version 130
#ifdef VERTEX_SHADER_SOURCE

uniform mat4 modelMat;
uniform mat4 mvMat;
uniform mat4 projMat;

in vec3 inVertex;
in vec2 inTexCoord;
in vec3 inColor;
in vec3 inNormal;
out vec4 color;
out vec2 texCoord;
out vec4 worldPosition;
out vec4 normal;

void main(void)
{
	mat4 finalMatrix = projMat * mvMat;
	gl_Position = finalMatrix * vec4(inVertex, 1.0);
	worldPosition = modelMat * vec4(inVertex, 1.0);
	color = vec4(inColor, 1.0);
	texCoord = inTexCoord;
	normal = modelMat * vec4(inNormal, 1.0);
}
#endif

#ifdef FRAGMENT_SHADER_SOURCE
uniform sampler2D texture0;
uniform vec3 sphereCenters[8];
uniform float sphereRadii[8];

in vec4 color;
in  vec2 texCoord;
in vec4 worldPosition;
in vec4 normal;
out vec4 outColor;

void main(void)
{
	outColor = vec4(0.0f, 0.0f, 0.0f, 0.0f);
	bool isInSphere = false;
	for(int i = 0; i < 8; i++)
	{
		if(length(worldPosition.xyz - sphereCenters[i]) < sphereRadii[i])
		{
			isInSphere = true;
		}
	}
	if(isInSphere)
		discard;
	else
	{
		vec3 lightDirection = vec3(1.0, 0.8, -1.1);
		lightDirection = normalize(lightDirection);
		if(gl_FrontFacing)
			outColor = color*texture(texture0, texCoord.st);
		else
		{
			outColor = color*textureLod(texture0, texCoord.st, 10);
		}

		vec3 ambient = 0.1 * outColor.rgb * vec3(1,1,1) * 1.0;
		float diffuseCoefficient = max(0.0, dot(normal.xyz, -lightDirection.xyz));
		vec3 diffuse = diffuseCoefficient * outColor.rgb * vec3(1,1,1) * 1.0;

		outColor.rgb = mix(ambient+diffuse, ambient, clamp(-worldPosition.z/20.0, 0.0, 1.0));
		vec3 gamma = vec3(0.76);
	    outColor = vec4(pow(outColor.rgb, gamma), outColor.a);
	}
}
#endif
