struct PSInput
{
	float4 position : SV_POSITION;
	float4 worldPosition : POSITION;
	float2 uv : TEXCOORD;
	float4 color : COLOR;
	float4 normal : NORMAL;
};

Texture2D g_texture : register(t0);
SamplerState g_sampler : register(s0);

cbuffer cb0 : register(b0)
{
	float4x4 modelMat;
	float4x4 mvMat;
	float4x4 projMat;
	float3 sphereCenters[8];
	float sphereRadii[8];
};

PSInput VSMain(float3 position : POSITION, float2 uv : TEXCOORD, float3 color : COLOR, float3 normal: NORMAL)
{
	PSInput result;

	float4x4 finalMatrix = mul(projMat, mvMat);
	result.position = mul(finalMatrix, float4(position, 1.0f));
	result.worldPosition = mul(modelMat, float4(position, 1.0f));
	result.uv = uv;
	result.color = float4(color, 1.0);
	result.normal = float4(normal, 1.0);

	return result;
}

float4 PSMain(PSInput input, bool isFrontFace: SV_IsFrontFace) : SV_TARGET
{
	float4 outColor = float4(0.0f, 0.0f, 0.0f, 0.0f);
	bool isInSphere = false;
	for(int i = 0; i < 8; i++)
	{
		if(length(input.worldPosition.xyz - sphereCenters[i]) < sphereRadii[i])
		{
			isInSphere = true;
		}
	}
	if(isInSphere)
		discard;
	else
	{
		float3 lightDirection = float3(1.0, 0.8, -1.1);
		lightDirection = normalize(lightDirection);
		if(isFrontFace)
			outColor = input.color * g_texture.Sample(g_sampler, input.uv);
		else
			outColor = input.color * g_texture.SampleLevel(g_sampler, input.uv, 10000);

		float3 ambient = 0.1 * outColor.rgb * float3(1.0f,1.0f,1.0f) * 1.0;
		float diffuseCoefficient = max(0.0, dot(input.normal.xyz, -lightDirection.xyz));
		float3 diffuse = diffuseCoefficient * outColor.rgb * float3(1.0f,1.0f,1.0f) * 1.0;

		outColor.rgb = lerp(ambient+diffuse, ambient, clamp(-input.worldPosition.z/20.0, 0.0, 1.0));
		float3 gamma = float3(0.76, 0.76, 0.76);
	    outColor = float4(pow(outColor.rgb, gamma), outColor.a);
	}
	return outColor;
}
