struct PSInput
{
	float4 position : SV_POSITION;
	float4 worldPosition : POSITION;
	float2 uv : TEXCOORD;
	float4 color : COLOR;
	float4 normal : NORMAL;
};

Texture2D g_texture : register(t0);
SamplerState g_sampler : register(s0);

cbuffer cb0 : register(b0)
{
	float4x4 modelMat;
	float4x4 mvMat;
	float4x4 projMat;
	float3 sphereCenters[8];
	float sphereRadii[8];
};

PSInput VSMain(float3 position : POSITION, float2 uv : TEXCOORD, float3 color : COLOR, float3 normal: NORMAL)
{
	PSInput result;

	float4x4 finalMatrix = mul(projMat, mvMat);
	result.position = mul(finalMatrix, float4(position, 1.0f));
	result.worldPosition = mul(modelMat, float4(position, 1.0f));
	result.uv = uv;
	result.color = float4(color[0], color[1], color[2], 1.0);
	result.normal = float4(normal[0], normal[1], normal[2], 1.0);

	return result;
}

float4 PSMain(PSInput input) : SV_TARGET
{
	float4 outColor = input.color * g_texture.Sample(g_sampler, input.uv);
	if(length(input.worldPosition.xyz - sphereCenters[0]) > sphereRadii[0])
	{
		discard;
	}
	//if(outColor.a < 0.3)
	//	discard;
	return outColor;
}
