struct PSInput
{
	float4 position : SV_POSITION;
	float4 worldPosition : POSITION;
};

Texture2D g_texture : register(t0);
SamplerState g_sampler : register(s0);

cbuffer cb0 : register(b0)
{
	float4x4 modelMat;
	float4x4 mvMat;
	float4x4 projMat;
	float3 sphereCenters[8];
	float sphereRadii[8];
};

PSInput VSMain(float3 position : POSITION, float2 uv : TEXCOORD, float3 color : COLOR, float3 normal: NORMAL)
{
	PSInput result;

	float4x4 finalMatrix = mul(projMat, mvMat);
	result.position = mul(finalMatrix, float4(position, 1.0f));
	result.worldPosition = mul(modelMat, float4(position, 1.0f));

	return result;
}

float4 PSMain(PSInput input) : SV_TARGET
{
	float4 outColor = float4(1.0f, 0.0f, 0.0f, 1.0f);
	float distance = length(input.worldPosition.xyz - sphereCenters[0]);
	if(distance < sphereRadii[0] + 0.1f * sphereRadii[0])
	{
		float ratio = clamp(((distance - sphereRadii[0]) / (0.1f * sphereRadii[0])), 0.0f, 1.0f);
		outColor = lerp(float4(0.0f, 0.0f, 0.0f, 1.0f), float4(1.0f, 0.0f, 0.0f, 1.0f), ratio);
	}
	else
		discard;
	//if(outColor.a < 0.3)
	//	discard;
	return outColor;
}
